@@ -50,6 +50,10 @@ SCHEMA_KEYS = tuple(
 # Names of keywords where the value is an object whose values are schemas.
 # Note that in some cases ("dependencies"), the value may be a list of strings.
 SCHEMA_OBJECT_KEYS = ("properties", "patternProperties", "dependencies")
+ALL_KEYWORDS = tuple(
+    ["$schema", *SCHEMA_KEYS, *SCHEMA_OBJECT_KEYS]
+    + sum((s.split() for _, s in TYPE_SPECIFIC_KEYS), [])
+)
 
 
 def next_down(val: float) -> float:
@@ -158,7 +162,18 @@ def upper_bound_instances(schema: Schema) -> float:
             if isinstance(mul, int):
                 return 1 + (upper - lower) % mul
             return 1 + (upper - lower)  # Non-integer mul can only reduce upper bound
-    # TODO: upper-bound array instances
+    if (
+        get_type(schema) == ["array"]
+        and isinstance(schema.get("items"), dict)
+        and schema.get("maxItems", math.inf) < 100  # type: ignore
+    ):
+        # For simplicity, we use the upper bound with replacement; while we could
+        # tighten this by considering uniqueItems it's not worth the extra code.
+        items_bound = upper_bound_instances(schema["items"])  # type: ignore
+        if items_bound < 100:
+            lo, hi = schema.get("minItems", 0), schema["maxItems"]
+            assert isinstance(lo, int) and isinstance(hi, int)
+            return sum(items_bound ** n for n in range(lo, hi + 1))
     return math.inf
 
 
@@ -695,14 +710,6 @@ def merged(schemas: List[Any]) -> Optional[Schema]:
                 return None
             out["additionalProperties"] = m
 
-        if "contains" in out and "contains" in s and out["contains"] != s["contains"]:
-            # OK, this is a tricky bit of logic.  If we are merging schemas with
-            # unequal "contains" keys, then we want to keep the *more* restrictive
-            # of the two as the contains key, so we can leverage that for generation.
-            contains = [out["contains"], s["contains"]]
-            contains.sort(key=upper_bound_instances)
-            out["contains"], _ = contains
-            out["allOf"] = out.get("allOf", []) + [{"contains": c} for c in contains]
         if "allOf" in out and "allOf" in s:
             # All our allOf schemas will be de-duplicated by canonicalise
             out["allOf"] += s.pop("allOf")
@@ -720,16 +727,25 @@ def merged(schemas: List[Any]) -> Optional[Schema]:
             & set(out)
         ):
             out[key] = max([out[key], s.pop(key)])
-        if isinstance(out.get("multipleOf"), int) and isinstance(
-            s.get("multipleOf"), int
-        ):
+        if "multipleOf" in out and "multipleOf" in s:
             x, y = s.pop("multipleOf"), out["multipleOf"]
-            out["multipleOf"] = x * y // math.gcd(x, y)
+            if isinstance(x, int) and isinstance(y, int):
+                out["multipleOf"] = x * y // math.gcd(x, y)
+            elif x != y:
+                ratio = max(x, y) / min(x, y)
+                if ratio == int(ratio):  # e.g. x=0.5, y=2
+                    out["multipleOf"] = max(x, y)
+                else:
+                    return None
 
         for k, v in s.items():
             if k not in out:
                 out[k] = v
-            elif out[k] != v:
+            elif out[k] != v and k in ALL_KEYWORDS:
+                # If non-validation keys like `title` or `description` don't match,
+                # that doesn't really matter and we'll just go with first we saw.
+                known = "$ref $schema anyOf oneOf items if not pattern contains"
+                assert k in known.split(), (k, out[k], v)
                 return None
         out = canonicalish(out)
         if out == FALSEY:
